from braintree.sub_merchant_account.address_details import AddressDetails
from braintree.sub_merchant_account.business_details import BusinessDetails
from braintree.sub_merchant_account.contact_details import ContactDetails
from braintree.sub_merchant_account.funding_details import FundingDetails
from braintree.sub_merchant_account.sub_merchant_account import SubMerchantAccount



from braintree.configuration import Configuration
from braintree.resource import Resource
from braintree.sub_merchant_account import ContactDetails,  BusinessDetails, FundingDetails

class SubMerchantAccount(Resource):
    class Status(object):
        Pending = "pending"

    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)
        self.business_details = BusinessDetails(attributes.get("business", {}))
        self.funding_details = FundingDetails(attributes.get("funding", {}))
        self.contacts = self._build_contacts(attributes)

    def __repr__(self):
        detail_list = [
            "id",
            "tos_accepted",
            "contacts",
            "business_details",
            "funding_details",
        ]

        return super(SubMerchantAccount, self).__repr__(detail_list)

    def _build_contacts(self, attributes):
        contacts = []
        for contact_attributes in attributes.get("contacts", []):
            contacts.append(ContactDetails(contact_attributes))

        return contacts

    @staticmethod
    def create(params={}):
        return Configuration.gateway().sub_merchant_account.create(params)

    @staticmethod
    def update(sub_merchant_account_id, params={}):
        return Configuration.gateway().sub_merchant_account.update(sub_merchant_account_id, params)



import braintree
from braintree.resource import Resource

class VenmoAccount(Resource):
    """
    A class representing Braintree Venmo accounts.
    """
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        if "subscriptions" in attributes:
            self.subscriptions = [braintree.subscription.Subscription(gateway, subscription) for subscription in self.subscriptions]



import braintree
from braintree.address import Address
from braintree.resource import Resource
from braintree.configuration import Configuration

class PaymentMethod(Resource):
    @staticmethod
    def create(params={}):
        return Configuration.gateway().payment_method.create(params)

    @staticmethod
    def find(payment_method_token):
        return Configuration.gateway().payment_method.find(payment_method_token)

    @staticmethod
    def update(payment_method_token, params):
        return Configuration.gateway().payment_method.update(payment_method_token, params)

    @staticmethod
    def delete(payment_method_token):
        return Configuration.gateway().payment_method.delete(payment_method_token)

    @staticmethod
    def create_signature():
        return PaymentMethod.signature("create")

    @staticmethod
    def signature(type):
        signature = [
            "billing_address_id",
            "cardholder_name",
            "customer_id",
            "cvv",
            "device_data",
            "device_session_id",
            "expiration_date",
            "expiration_month",
            "expiration_year",
            "number",
            "payment_method_nonce",
            "token",
            {"billing_address": Address.create_signature()},
            {"options": [
                "fail_on_duplicate_payment_method",
                "make_default",
                "verification_merchant_account_id",
                "verify_card",
                ]
            }
        ]
        return signature

    @staticmethod
    def update_signature():
        signature = [
            "billing_address_id",
            "cardholder_name",
            "cvv",
            "device_session_id",
            "expiration_date",
            "expiration_month",
            "expiration_year",
            "number",
            "token",
            "venmo_sdk_payment_method_code",
            "device_data",
            "fraud_merchant_id",
            "payment_method_nonce",
            {"options": [
                "make_default",
                "verify_card",
                "verification_merchant_account_id",
                "venmo_sdk_session"
                ]
            },
            {"billing_address" :
                Address.update_signature() +
                [{"options": ["update_existing"]}]
            }
        ]
        return signature




import braintree
from braintree.resource import Resource

class AmexExpressCheckoutCard(Resource):
    """
    A class representing Braintree Amex Express Checkout card objects.
    """
    def __init__(self, gateway, attributes):
        Resource.__init__(self, gateway, attributes)

        if "subscriptions" in attributes:
            self.subscriptions = [braintree.subscription.Subscription(gateway, subscription) for subscription in self.subscriptions]

    @property
    def expiration_date(self):
        return self.expiration_month + "/" + self.expiration_year




from braintree.util import Constants
from braintree import Subscription
from braintree.search import Search

class SubscriptionSearch:
    billing_cycles_remaining = Search.RangeNodeBuilder("billing_cycles_remaining")
    days_past_due = Search.RangeNodeBuilder("days_past_due")
    id = Search.TextNodeBuilder("id")
    ids = Search.MultipleValueNodeBuilder("ids")
    in_trial_period = Search.MultipleValueNodeBuilder("in_trial_period")
    merchant_account_id = Search.MultipleValueNodeBuilder("merchant_account_id")
    next_billing_date = Search.RangeNodeBuilder("next_billing_date")
    plan_id = Search.MultipleValueOrTextNodeBuilder("plan_id")
    price = Search.RangeNodeBuilder("price")
    status = Search.MultipleValueNodeBuilder("status", Constants.get_all_constant_values_from_class(Subscription.Status))
    transaction_id = Search.TextNodeBuilder("transaction_id")



import braintree
from braintree.discount import Discount
from braintree.resource_collection import ResourceCollection

class DiscountGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def all(self):
        response = self.config.http().get(self.config.base_merchant_path() + "/discounts/")
        discounts = {"discount": response["discounts"]}
        return [Discount(self.gateway, item) for item in ResourceCollection._extract_as_array(discounts, "discount")]



from braintree.attribute_getter import AttributeGetter
from braintree.sub_merchant_account.address_details import AddressDetails

class BusinessDetails(AttributeGetter):
    detail_list = [
        "address_details",
        "dba_name",
        "legal_name",
        "registered_as",
        "registration_number",
        "tax_id",
        "vat",
    ]

    def __init__(self, attributes):
        AttributeGetter.__init__(self, attributes)
        self.address_details = AddressDetails(attributes.get("address", {}))

    def __repr__(self):
        return super(BusinessDetails, self).__repr__(self.detail_list)



from braintree.exceptions.braintree_error import BraintreeError

class AuthenticationError(BraintreeError):
    """
    Raised when the client library cannot authenticate with the gateway.  This generally means the public_key/private key are incorrect, or the user is not active.

    See https://developers.braintreepayments.com/ios+python/reference/general/exceptions#authentication-error
    """
    pass



from braintree.error_result import ErrorResult
from braintree.sub_merchant_account import SubMerchantAccount
from braintree.resource import Resource
from braintree.successful_result import SuccessfulResult
from braintree.exceptions.not_found_error import NotFoundError

class SubMerchantAccountGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def create(self, params={}):
        Resource.verify_keys(params, SubMerchantAccountGateway._create_signature())
        return self._post("/sub_merchant_accounts", {"sub_merchant_account": params})

    def update(self, sub_merchant_account_id, params={}):
        Resource.verify_keys(params, SubMerchantAccountGateway._update_signature())
        return self._put("/sub_merchant_accounts/" + sub_merchant_account_id, {"sub_merchant_account": params})

    def _post(self, url, params={}):
        response = self.config.http().post(self.config.base_merchant_path() + url, params)
        if "sub_merchant_account" in response:
            return SuccessfulResult({"sub_merchant_account": SubMerchantAccount(self.gateway, response["sub_merchant_account"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])
        else:
            pass

    def _put(self, url, params={}):
        response = self.config.http().put(self.config.base_merchant_path() + url, params)
        if "sub_merchant_account" in response:
            return SuccessfulResult({"sub_merchant_account": SubMerchantAccount(self.gateway, response["sub_merchant_account"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])
        else:
            pass

    @staticmethod
    def _create_signature():
        return [
            {'business': [
                {'address': [
                    'country',
                    'locality',
                    'postal_code',
                    'region',
                    'street_address',
                    ],
                },
                'dba_name',
                'legal_name',
                'registered_as',
                'registration_number',
                'tax_id',
                'vat',
                ],
            },
            {'contacts': [
                {'address': [
                    'country',
                    'locality',
                    'postal_code',
                    'region',
                    'street_address',
                    ],
                },
                'date_of_birth',
                'email',
                'first_name',
                'last_name',
                'phone',
                ]
            },
            {'funding': [
                'account_holder_name',
                'bic',
                'currency_iso_code',
                'descriptor',
                'iban',
                ]
            },
            'id',
            'tos_accepted',
            'verify_identity',
        ]

    @staticmethod
    def _update_signature():
        return [
            {'business': [
                {'address': [
                    'country',
                    'locality',
                    'postal_code',
                    'region',
                    'street_address',
                    ],
                },
                'dba_name',
                'legal_name',
                'registered_as',
                'registration_number',
                'tax_id',
                'vat',
                ],
            },
            {'contacts': [
                {'address': [
                    'country',
                    'locality',
                    'postal_code',
                    'region',
                    'street_address',
                    ],
                },
                'date_of_birth',
                'email',
                'first_name',
                'id',
                'last_name',
                'phone',
                ]
            },
            {'funding': [
                'account_holder_name',
                'bic',
                'currency_iso_code',
                'descriptor',
                'iban',
                ]
            },
            'id',
            'tos_accepted',
            'verify_identity',
        ]



from braintree.attribute_getter import AttributeGetter

class FundingDetails(AttributeGetter):
    detail_list = [
        "account_holder_name",
        "bic",
        "currency_iso_code",
        "descriptor",
        "iban",
    ]

    def __init__(self, attributes):
        AttributeGetter.__init__(self, attributes)

    def __repr__(self):
        return super(FundingDetails, self).__repr__(self.detail_list)



import braintree
from braintree.error_result import ErrorResult
from braintree.resource import Resource
from braintree.resource_collection import ResourceCollection
from braintree.successful_result import SuccessfulResult
from braintree.transaction import Transaction
from braintree.transparent_redirect import TransparentRedirect
from braintree.exceptions.not_found_error import NotFoundError
from braintree.exceptions.down_for_maintenance_error import DownForMaintenanceError

class TransactionGateway(object):
    def __init__(self, gateway):
        self.gateway = gateway
        self.config = gateway.config

    def clone_transaction(self, transaction_id, params):
        Resource.verify_keys(params, Transaction.clone_signature())
        return self._post("/transactions/" + transaction_id + "/clone", {"transaction-clone": params})

    def cancel_release(self, transaction_id):
        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/cancel_release", {})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def confirm_transparent_redirect(self, query_string):
        id = self.gateway.transparent_redirect._parse_and_validate_query_string(query_string)["id"][0]
        return self._post("/transactions/all/confirm_transparent_redirect_request", {"id": id})

    def create(self, params):
        Resource.verify_keys(params, Transaction.create_signature())
        return self._post("/transactions", {"transaction": params})

    def find(self, transaction_id):
        try:
            if transaction_id is None or transaction_id.strip() == "":
                raise NotFoundError()
            response = self.config.http().get(self.config.base_merchant_path() + "/transactions/" + transaction_id)
            return Transaction(self.gateway, response["transaction"])
        except NotFoundError:
            raise NotFoundError("transaction with id " + repr(transaction_id) + " not found")

    def hold_in_escrow(self, transaction_id):
        """
        Holds an existing submerchant transaction for escrow. It expects a transaction_id. ::

            result = braintree.Transaction.hold_in_escrow("my_transaction_id")
        """

        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/hold_in_escrow", {})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def refund(self, transaction_id, amount_or_options=None):
        """
        Refunds an existing transaction. It expects a transaction_id. ::

            result = braintree.Transaction.refund("my_transaction_id")
        """
        if isinstance(amount_or_options, dict):
            options = amount_or_options
        else:
            options = {
                "amount": amount_or_options
            }
        Resource.verify_keys(options, Transaction.refund_signature())
        response = self.config.http().post(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/refund", {"transaction": options})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def sale(self, params):
        params.update({"type": "sale"})
        return self.create(params)

    def search(self, *query):
        if isinstance(query[0], list):
            query = query[0]

        response = self.config.http().post(self.config.base_merchant_path() + "/transactions/advanced_search_ids", {"search": self.__criteria(query)})
        if "search_results" in response:
            return ResourceCollection(query, response, self.__fetch)
        else:
            raise DownForMaintenanceError("search timeout")

    def release_from_escrow(self, transaction_id):
        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/release_from_escrow", {})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def submit_for_settlement(self, transaction_id, amount=None, params={}):
        Resource.verify_keys(params, Transaction.submit_for_settlement_signature())
        transaction_params = {"amount": amount}
        transaction_params.update(params)
        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/submit_for_settlement",
                {"transaction": transaction_params})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def update_details(self, transaction_id, params={}):
        Resource.verify_keys(params, Transaction.update_details_signature())
        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/update_details",
                {"transaction": params})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def submit_for_partial_settlement(self, transaction_id, amount, params={}):
        Resource.verify_keys(params, Transaction.submit_for_settlement_signature())
        transaction_params = {"amount": amount}
        transaction_params.update(params)
        response = self.config.http().post(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/submit_for_partial_settlement",
                {"transaction": transaction_params})
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def tr_data_for_credit(self, tr_data, redirect_url):
        if "transaction" not in tr_data:
            tr_data["transaction"] = {}
        tr_data["transaction"]["type"] = Transaction.Type.Credit
        Resource.verify_keys(tr_data, [{"transaction": Transaction.create_signature()}])
        tr_data["kind"] = TransparentRedirect.Kind.CreateTransaction
        return self.gateway.transparent_redirect.tr_data(tr_data, redirect_url)

    def tr_data_for_sale(self, tr_data, redirect_url):
        if "transaction" not in tr_data:
            tr_data["transaction"] = {}
        tr_data["transaction"]["type"] = Transaction.Type.Sale
        Resource.verify_keys(tr_data, [{"transaction": Transaction.create_signature()}])
        tr_data["kind"] = TransparentRedirect.Kind.CreateTransaction
        return self.gateway.transparent_redirect.tr_data(tr_data, redirect_url)

    def transparent_redirect_create_url(self):
        return self.config.base_url() + self.config.base_merchant_path() + "/transactions/all/create_via_transparent_redirect_request"

    def void(self, transaction_id):
        response = self.config.http().put(self.config.base_merchant_path() + "/transactions/" + transaction_id + "/void")
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])

    def __fetch(self, query, ids):
        criteria = self.__criteria(query)
        criteria["ids"] = braintree.transaction_search.TransactionSearch.ids.in_list(ids).to_param()
        response = self.config.http().post(self.config.base_merchant_path() + "/transactions/advanced_search", {"search": criteria})
        return [Transaction(self.gateway, item) for item in  ResourceCollection._extract_as_array(response["credit_card_transactions"], "transaction")]

    def __criteria(self, query):
        criteria = {}
        for term in query:
            if criteria.get(term.name):
                criteria[term.name] = dict(list(criteria[term.name].items()) + list(term.to_param().items()))
            else:
                criteria[term.name] = term.to_param()
        return criteria

    def _post(self, url, params={}):
        response = self.config.http().post(self.config.base_merchant_path() + url, params)
        if "transaction" in response:
            return SuccessfulResult({"transaction": Transaction(self.gateway, response["transaction"])})
        elif "api_error_response" in response:
            return ErrorResult(self.gateway, response["api_error_response"])




from braintree.attribute_getter import AttributeGetter

class AddressDetails(AttributeGetter):
    detail_list = [
        "country",
        "locality",
        "postal_code",
        "region",
        "street_address",
    ]

    def __init__(self, attributes):
        AttributeGetter.__init__(self, attributes)

    def __repr__(self):
        return super(AddressDetails, self).__repr__(self.detail_list)
